'' =================================================================================================
''
''   File....... hc-8_remote_inputs_outputs_test.spin
''   Purpose.... Setup for multi-cog access of digital inputs
''               -- digital inputs auto-scanned
''   Author..... William "Bsnut" Stefan
''               Copyright (c) 2014-2015 
''               All Rights Reserved
''   E-mail..... 
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000
  

con

  PCB_REV = "C"                                                 ' production boards

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28

  OUT_EN  = 27                                                  ' output enable     
  
  SIO     = 26                                                  ' TTL serial io

  NC25    = 25                                                  ' not connected
  NC24    = 24
  
  RX2     = 23                                                  ' RS-485 interface
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  NC19    = 19
  
  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  NC16    = 16
  
  OUT7    = 15                                                  ' channel outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' option inputs
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' DMX address.8 input

  DO_165  =  2                                                  ' 74x165 IO
  CLK_165 =  1 
  LD_165  =  0


  ' for EFX-TEK uSD adapter module
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM  


con

   #0, NO, YES
   #0, OFF, RED, GRN

   #0, NOCON, RX, TX, RXTX

   #0, LSBFIRST, MSBFIRST

   USE_DIGINS = YES                                             ' set to 0 to disable
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS




 


obj

  com1  : "fullduplexserial64"


var
  
  
dat
'string used by "pub hc8_set_digital" and "pub hc8_digital_select"
                                   

HC8             byte    "!HC8", 0

pub main 

  setup_io                                                      ' setup HC-8+ IO pins
  com1.start(SIO, SIO, %1100, 38_400)   'half-duplex for HC-8+ comms

  ' start other cogs here
  
  pause(1)                                                      ' let all cogs start




  repeat
    if ttl_bit(0) == 1      '   
      hc8_set_digital(%00, 4, 1)
    else
      hc8_set_digital(%00, 4, 0)

    if hc8_inputs_bit(%00, 1) 
      high(OUT3)
    else
      low(OUT3)  

    pause(100)                        ' run loop every 100ms


con

  { ================================= }
  {                                   }
  {  S U P P O R T   R O U T I N E S  } 
  {                                   }
  { ================================= }


pub hc8_set_digital(addr, ch, state)  ' Sends commands to the Non-Hack HC-8+

  com1.str(@HC8)
  com1.tx(addr)
  com1.tx("D")
  com1.tx(ch)
  com1.tx(state)


pub hc8_digital_select(addr, states, mask) ' Sends commands to the Non-Hack HC-8+

   com1.str(@HC8)
  com1.tx(addr)
  com1.tx("D")
  com1.tx("S")  
  com1.tx(states)
  com1.tx(mask)


pub hc8_inputs(addr)

'' Read ttl inputs from remote HC-8+ at address

  com1.str(string("!HC8"))
  com1.tx(addr)
  com1.tx("I") 
  com1.rxflush

  return (com1.rxtime(25) #> 0)

  
pub hc8_inputs_bit(addr, bit) | remotettlbit 

'' Returns 1 or 0 status of the remote HC-8 TTL bits
'' "addr" is the address of the remote HC-8  
'' "bit" is the remote TTL bit. Valid bits are 0 to 7


  com1.str(string("!HC8"))
  com1.tx(addr)
  com1.tx("I")
  com1.rxflush
  remotettlbit := com1.rxtime(25)

  if ((bit => 0) and (bit =< 7))                             ' valid bit?
    return (remotettlbit & (1 << bit)) >> bit                ' extract bit
  else
    return 0

  
pub setup_io

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245 

  set_rs485(NOCON)                                              ' RS-485 off

  ' launch input scanner cog ONLY if using TTL or DMX address inputs
  
  if (USE_DIGINS == YES)                                   ' use digital inputs
    cognew(scan_digins, @digstack)                         ' yes, launch scanner
         

pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

      
pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin, refreshes output mode

  !outa[pin]
  dira[pin] := 1
  

pub input(pin)

'' Set pin to input mode and read back current state

  dira[pin] := 0

  return ina[pin]


pub set_rgled(state)

'' Sets R/G LED

  case state
    OFF:
      outa[R_LED..G_LED] := %00
      
    RED:
      outa[R_LED..G_LED] := %10
      
    GRN:
      outa[R_LED..G_LED] := %01
                        
  dira[R_LED..G_LED] := %11


pub set_rs485(state)

'' Sets RS-485 RX and TX enable pins

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11


pub option_inputs

'' Returns 4-bit value of OPTION dipswitch inputs
'' -- "1" bit = ON
'' -- bit order: BR (3), A1 (2), A0 (1), SM (0)

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode

  return !ina[OPT_BR..OPT_SM] & %1111                           ' return bits



con

  { ============================= }
  {                               }
  {  D I G I T A L   I N P U T S  }
  {                               } 
  { ============================= } 


var

  long  digins                                                  ' TTL & DMX addr inputs
  long  digstack[16]                                            ' stack for digins cog
  
  
pub ttl_inputs 

'' Return TTL inputs status
 
  return (digins & $FF)                                         ' return TTL input bits


pub dmx_address 

'' Return DMX address switch setting

  return (digins >> 8) & $01FF                                  ' return DMX address bits
 

pub digin_bit(bit)

'' Returns 1 or 0
'' -- valid bits are 0 to 16
'' -- TTL bits are 0 to 7
'' -- DMX address bits are 8 to 16

  if ((bit => 0) and (bit =< 16))                               ' valid bit?
    return (digins & (1 << bit)) >> bit                         ' extract bit
  else
    return 0


pub ttl_bit(bit)

'' Returns specific bit of TTL inputs group
'' -- valid bits are 0 to 7

  if ((bit => 0) and (bit =< 7))                                ' valid bit?
    return digin_bit(bit)                                       ' return bit
  else
    return 0


pub dmx_bit(bit)

'' Returns specific bit of DMX address switch 
'' -- valid bits are 0 to 8

  if ((bit => 0) and (bit =< 8))                                ' valid bit?
    return digin_bit(bit + 8)                                   ' offset for dmx switch
  else
    return 0
    
  
pri scan_digins | t, work                                       ' launch with cognew

'' Scans TTL and DMX address inputs
'' -- uses two 74x165 shift registers
'' -- (debounced) global inputs value updated every 25ms

  high(LD_165)                                                  ' setup x165 io pins
  low(CLK_165)

  t := cnt
  repeat
    work := 0                                                   ' clear workspace
    repeat 25                                                   ' debounce 25ms
      outa[LD_165] := 0                                         ' load inputs   
      outa[LD_165] := 1                                          
      repeat 16                                                 ' read 16 bits         
        work := (work << 1) | ina[DO_165]                       ' get new bit             
        outa[CLK_165] := 1                                      ' blip clock              
        outa[CLK_165] := 0                                       
      work |= (ina[DMX_A8] << 16)                               ' add DMX.A8 bit                       
      waitcnt(t += (MS_001))                                    ' refresh every 1ms

    digins := work                                              ' update debounced input   

  
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      